@@ -27,7 +27,6 @@
 namespace kiwix
 {
 
-class Searcher;
 class NameMapper;
 /**
  * The SearcherRenderer class is used to render a search result to a html page.
@@ -35,17 +34,6 @@ class NameMapper;
 class SearchRenderer
 {
  public:
-  /**
-   * Construct a SearchRenderer from a Searcher.
-   *
-   * This method is now deprecated. Construct the renderer from a
-   * `zim::SearchResultSet`
-   *
-   * @param searcher The `Searcher` to render.
-   * @param mapper The `NameMapper` to use to do the rendering.
-   */
-  DEPRECATED SearchRenderer(Searcher* searcher, NameMapper* mapper);
-
   /**
    * Construct a SearchRenderer from a SearchResultSet.
    *
@@ -75,12 +63,15 @@ class SearchRenderer
 
   ~SearchRenderer();
 
+  /**
+   * Set the search pattern used to do the search
+   */
   void setSearchPattern(const std::string& pattern);
 
   /**
-   * Set the search content id.
+   * Set the querystring used to select books
    */
-  void setSearchContent(const std::string& name);
+  void setSearchBookQuery(const std::string& bookQuery);
 
   /**
    * Set protocol prefix.
@@ -99,17 +90,25 @@ class SearchRenderer
     this->pageLength  = pageLength;
   }
 
+  std::string renderTemplate(const std::string& tmpl_str);
+
   /**
    * Generate the html page with the resutls of the search.
    */
   std::string getHtml();
 
+    /**
+   * Generate the xml page with the resutls of the search.
+   */
+  std::string getXml();
+
+
  protected:
   std::string beautifyInteger(const unsigned int number);
   zim::SearchResultSet m_srs;
   NameMapper* mp_nameMapper;
   Library* mp_library;
-  std::string searchContent;
+  std::string searchBookQuery;
   std::string searchPattern;
   std::string protocolPrefix;
   std::string searchProtocolPrefix;
