@@ -1,4 +1,12 @@
 (function() {
+    class FragmentParams extends URLSearchParams {
+        constructor(fragment = '') {
+            if (fragment[0] == '#')
+                fragment = fragment.substring(1);
+            super(fragment);
+        }
+    }
+
     const root = document.querySelector(`link[type='root']`).getAttribute('href');
     const incrementalLoadingParams = {
         start: 0,
@@ -14,21 +22,49 @@
     let isFetching = false;
     let noResultInjected = false;
     let filters = getCookie(filterCookieName);
-    let params = new URLSearchParams(window.location.search || filters || '');
+    let params = new FragmentParams(window.location.hash || filters || '');
+    params.delete('userlang');
     let timer;
     let languages = {};
+    let previousScrollTop = Infinity;
+
+    function updateFeedLink() {
+        const inputParams = new FragmentParams(window.location.hash);
+        const filteredParams = new FragmentParams();
+        for (const [key, value] of inputParams) {
+            if ( value != '' ) {
+                filteredParams.set(key, value);
+            }
+        }
+        const feedLink = `${root}/catalog/v2/entries?${filteredParams.toString()}`;
+        document.querySelector('#headFeedLink').href = feedLink;
+        document.querySelector('#feedLink').href = feedLink;
+        setFeedToolTip();
+    }
+
+    function changeUILanguage() {
+      window.modalUILanguageSelector.close();
+      const s = document.getElementById("ui_language");
+      const lang = s.options[s.selectedIndex].value;
+      setPermanentGlobalCookie('userlang', lang);
+      window.location.reload();
+    }
 
     function queryUrlBuilder() {
-        let url = `${root}/catalog/search?`;
+        let url = `${root}/catalog/v2/entries?`;
         url += Object.keys(incrementalLoadingParams).map(key => `${key}=${incrementalLoadingParams[key]}`).join("&");
         params.forEach((value, key) => {url+= value ? `&${key}=${value}` : ''});
         return (url);
     }
 
-    function setCookie(cookieName, cookieValue) {
-        const date = new Date();
-        date.setTime(date.getTime() + oneDayDelta);
-        document.cookie = `${cookieName}=${cookieValue};expires=${date.toUTCString()};sameSite=Strict`;
+    function setCookie(cookieName, cookieValue, ttl) {
+        let exp = "";
+        if ( ttl ) {
+          const date = new Date();
+          date.setTime(date.getTime() + ttl);
+          exp = `expires=${date.toUTCString()};`;
+        }
+        document.cookie = `${cookieName}=${cookieValue};${exp}sameSite=Strict`;
     }
 
     function getCookie(cookieName) {
@@ -80,7 +116,7 @@
     function generateTagLink(tagValue) {
         tagValue = tagValue.toLowerCase();
         const humanFriendlyTagValue = humanFriendlyTitle(tagValue);
-        const tagMessage = `Filter by tag "${humanFriendlyTagValue}"`;
+        const tagMessage = $t("filter-by-tag", {TAG: humanFriendlyTagValue});
         return `<span class='tag__link' aria-label='${tagMessage}' title='${tagMessage}' data-tag=${tagValue}>${humanFriendlyTagValue}</span>`
     }
 
@@ -95,8 +131,13 @@
         const title =  getInnerHtml(book, 'title');
         const description = getInnerHtml(book, 'summary');
         const id = getInnerHtml(book, 'id');
-        const langCode = getInnerHtml(book, 'language');
-        const language = languages[langCode];
+        const langCodesList = getInnerHtml(book, 'language').split(',');
+        const langCode = langCodesList.length == 1 ? langCodesList[0] : 'mul';
+        let language = languages[langCode];
+        if (langCode == 'mul') {
+            const mulLangList = langCodesList.filter(x => languages.hasOwnProperty(x)).map(x => languages[x]);
+            language = mulLangList.join(', ');
+        }
         const tags = getInnerHtml(book, 'tags');
         const tagList = tags.split(';').filter(tag => {return !(tag.startsWith('_'))});
         const tagFilterLinks = tagList.map((tagValue) => generateTagLink(tagValue));
@@ -130,7 +171,7 @@
             <div class="book__icon" ${faviconAttr}></div>
             <div class="book__header">
                 <div id="book__title">${title}</div>
-                ${downloadLink ? `<div class="book__download"><span data-link="${downloadLink}">Download ${humanFriendlyZimSize ? ` - ${humanFriendlyZimSize}</span></div>`: ''}` : ''}
+                ${downloadLink ? `<div class="book__download"><span data-link="${downloadLink}">${$t("download")} ${humanFriendlyZimSize ? ` - ${humanFriendlyZimSize}</span></div>`: ''}` : ''}
             </div>
             <div class="book__description" title="${description}">${description}</div>
             </div>
@@ -196,27 +237,27 @@
                     <div class="modal-content">
                         <div class="modal-regular-download">
                             <a href="${downloadLink}" download>
-                                <img src="../skin/download.png?KIWIXCACHEID" alt="direct download" />
-                                <div>Direct</div>
+                                <img src="${root}/skin/download.png?KIWIXCACHEID" alt="${$t("direct-download-alt-text")}" />
+                                <div>${$t("direct-download-link-text")}</div>
                             </a>
                         </div>
                         <div class="modal-regular-download">
                             <a href="${downloadLink}.sha256" download>
-                                <img src="../skin/hash.png?KIWIXCACHEID" alt="download hash" />
-                                <div>Sha256 hash</div>
+                                <img src="${root}/skin/hash.png?KIWIXCACHEID" alt="${$t("hash-download-alt-text")}" />
+                                <div>${$t("hash-download-link-text")}</div>
                             </a>
                         </div>
                         ${magnetLink ?
                         `<div class="modal-regular-download">
                             <a href="${magnetLink}" target="_blank">
-                                <img src="../skin/magnet.png?KIWIXCACHEID" alt="download magnet" />
-                                <div>Magnet link</div>
+                                <img src="${root}/skin/magnet.png?KIWIXCACHEID" alt="${$t("magnet-alt-text")}" />
+                                <div>${$t("magnet-link-text")}</div>
                             </a>
                         </div>` : ``}
                         <div class="modal-regular-download">
                             <a href="${downloadLink}.torrent" download>
-                                <img src="../skin/bittorrent.png?KIWIXCACHEID" alt="download torrent" />
-                                <div>Torrent file</div>
+                                <img src="${root}/skin/bittorrent.png?KIWIXCACHEID" alt="${$t("torrent-download-alt-text")}" />
+                                <div>${$t("torrent-download-link-text")}</div>
                             </a>
                         </div>
                     </div>
@@ -226,7 +267,7 @@
     }
 
     async function getBookCount(query) {
-        const url = `${root}/catalog/search?${query}`;
+        const url = `${root}/catalog/v2/entries?${query}&count=0`;
         return await fetch(url).then(async (resp) => {
             const data = new window.DOMParser().parseFromString(await resp.text(), 'application/xml');
             return parseInt(data.querySelector('totalResults').innerHTML);
@@ -249,16 +290,10 @@
             } else {
                 toggleFooter();
             }
-            const kiwixResultText = document.querySelector('.kiwixHomeBody__results')
-            if (results) {
-                let resultText = `${results} books`;
-                if (results === 1) {
-                    resultText = `${results} book`;
-                }
-                kiwixResultText.innerHTML = resultText;
-            } else {
-                kiwixResultText.innerHTML = ``;
-            }
+            const text = results
+                       ? $t("count-of-matching-books", {COUNT: results})
+                       : '';
+            document.querySelector('.kiwixHomeBody__results').innerHTML = text;
             loader.style.display = 'none';
             return books;
         });
@@ -285,7 +320,7 @@
         const kiwixHomeBody = document.querySelector('.kiwixHomeBody');
         const divTag = document.createElement('div');
         divTag.setAttribute('class', 'noResults');
-        divTag.innerHTML = `No result. Would you like to <a href="?lang=">reset filter</a>?`;
+        divTag.innerHTML = $t("welcome-page-overzealous-filter", {URL: '#lang='});
         kiwixHomeBody.append(divTag);
         kiwixHomeBody.setAttribute('style', 'display: flex; justify-content: center; align-items: center');
         loader.setAttribute('style', 'position: absolute; top: 50%');
@@ -356,13 +391,14 @@
         incrementalLoadingParams.count = viewPortToCount();
         fadeOutDiv.style.display = 'none';
         bookOrderMap.clear();
-        params = new URLSearchParams(window.location.search);
+        params = new FragmentParams(window.location.hash);
         if (filterType) {
             params.set(filterType, filterValue);
-            window.history.pushState({}, null, `?${params.toString()}`);
-            setCookie(filterCookieName, params.toString());
+            window.history.pushState({}, null, `#${params.toString()}`);
+            setCookie(filterCookieName, params.toString(), oneDayDelta);
         }
         updateFilterColors();
+        updateFeedLink();
         await loadAndDisplayBooks(true);
     }
 
@@ -397,7 +433,7 @@
         tagElement.style.display = 'inline-block';
         const humanFriendlyTagValue = humanFriendlyTitle(tagValue);
         tagElement.innerHTML = `${humanFriendlyTagValue}`;
-        const tagMessage = `Stop filtering by tag "${humanFriendlyTagValue}"`;
+        const tagMessage = $t("stop-filtering-by-tag", {TAG: humanFriendlyTagValue});
         tagElement.setAttribute('aria-label', tagMessage);
         tagElement.setAttribute('title', tagMessage);
         if (resetFilter)
@@ -432,6 +468,22 @@
         }
     }
 
+    function updateNavVisibilityState() {
+        const st = window.scrollY;
+        const enableAutoHiding = document.body.clientWidth < 590;
+        if ((Math.abs(previousScrollTop - st) <= 5) || !enableAutoHiding)
+            return;
+        const kiwixNav = document.querySelector('.kiwixNav');
+        if (st > previousScrollTop) {
+            kiwixNav.style.position = 'fixed';
+            kiwixNav.style.top = '-100%';
+        } else {
+            kiwixNav.style.position = 'sticky';
+            kiwixNav.style.top = '0';
+        }
+        previousScrollTop = st;
+    }
+
     window.addEventListener('resize', (event) => {
         if (timer) {clearTimeout(timer)}
         timer = setTimeout(() => {
@@ -448,7 +500,42 @@
         }
     });
 
-    window.onload = async () => {
+    window.addEventListener('hashchange', () => resetAndFilter());
+
+    function setFeedToolTip() {
+        const feedLogoElem = document.getElementById('feedLogo');
+        const libraryOpdsFeedHint = opdsFeedHintByParams();
+        for (const attr of ["alt", "aria-label", "title"] ) {
+            feedLogoElem.setAttribute(attr, libraryOpdsFeedHint);
+        }
+    }
+
+    function opdsFeedHintByParams() {
+        const paramObj = {};
+        const inputParams = new FragmentParams(window.location.hash);
+        for (const [key, value] of inputParams) {
+            if ( value != '' ) {
+                paramObj[key.toUpperCase()] = value;
+            }
+        }
+        if (!paramObj.LANG && !paramObj.CATEGORY && !paramObj.TAG && !paramObj.Q) {
+            return $t('library-opds-feed-all-entries');
+        }
+        return $t('library-opds-feed-parameterised', paramObj);
+    }
+
+    function updateUIText() {
+        footer.innerHTML = $t("powered-by-kiwix-html");
+        const searchText = $t("search");
+        document.getElementById('searchFilter').placeholder = searchText;
+        document.getElementById('searchButton').value = searchText;
+        document.getElementById('categoryFilter').children[0].innerHTML = $t("book-filtering-all-categories");
+        document.getElementById('languageFilter').children[0].innerHTML = $t("book-filtering-all-languages");
+        setFeedToolTip();
+    }
+
+    async function onload() {
+        initUILanguageSelector(getUserLanguage(), changeUILanguage);
         iso = new Isotope( '.book__list', {
             itemSelector: '.book',
             getSortData:{
@@ -464,6 +551,7 @@
             }
         });
         footer = document.getElementById('kiwixfooter');
+        updateUIText();
         fadeOutDiv = document.getElementById('fadeOut');
         loader = document.querySelector('.loader');
         await loadAndDisplayOptions('#languageFilter', `${root}/catalog/v2/languages`, 'language');
@@ -475,15 +563,15 @@
         const tagElement = document.getElementsByClassName('tagFilterLabel')[0];
         tagElement.addEventListener('click', () => removeTagElement(true));
         if (filters) {
-            const currentLink = window.location.search;
-            const newLink = `?${params.toString()}`;
+            const currentLink = window.location.hash;
+            const newLink = `#${params.toString()}`;
             if (currentLink != newLink) {
                 window.history.pushState({}, null, newLink);
             }
         }
         updateVisibleParams();
         document.getElementById('kiwixSearchForm').onsubmit = (event) => {event.preventDefault()};
-        if (!window.location.search) {
+        if (!window.location.hash) {
             const browserLang = navigator.language.split('-')[0];
             const langFilter = document.getElementById('languageFilter');
             const lang = browserLang.length === 3 ? browserLang : iso6391To3[browserLang];
@@ -492,7 +580,16 @@
                 langFilter.dispatchEvent(new Event('change'));
             }
         }
-        setCookie(filterCookieName, params.toString());
+        updateFeedLink();
+        setCookie(filterCookieName, params.toString(), oneDayDelta);
+        setInterval(updateNavVisibilityState, 250);
+    };
+
+    // required by i18n.js:setUserLanguage()
+    window.setPermanentGlobalCookie = function(name, value) {
+        document.cookie = `${name}=${value};path=${root};max-age=31536000`;
     }
+
+    window.onload = () => { setUserLanguage(getUserLanguage(), onload); }
 })();
 
